<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter;
use TYPO3\Election\Domain\Model\Configuration;

/**
 * Class AbstractBeController
 */
abstract class AbstractBeController extends ActionController
{
    /**
     * @var BackendUserAuthentication
     */
    protected $backendUser = null;

    /**
     * @var Configuration
     */
    protected $configuration = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ConfigurationRepository
     * @inject
     */
    protected $configurationRepository = null;

    /**
     * Sets the current configuration
     */
    public function initializeObject()
    {
        $this->configuration = $this->configurationRepository->findCurrent();
    }

    /**
     * ModuleController constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this->backendUser = self::getBackendUserAuthentication();
    }

    /**
     * @param ViewInterface $view
     */
    public function initializeView(ViewInterface $view)
    {
        $view->assign('backendUserData', $this->backendUser->user);
        $view->assign('configuration', $this->configuration);
    }

    /**
     * @return BackendUserAuthentication
     * @SuppressWarnings(PHPMD.Superglobals)
     */
    protected static function getBackendUserAuthentication()
    {
        return $GLOBALS['BE_USER'];
    }

    /**
     * @param string $messageBody
     * @param string $messageTitle
     * @param int $severity
     * @param bool $storeInSession
     */
    public function addFlashMessage(
        $messageBody,
        $messageTitle = '',
        $severity = AbstractMessage::OK,
        $storeInSession = true
    ) {
        if (null === $this->controllerContext) {
            $this->controllerContext = $this->buildControllerContext();
        }
        parent::addFlashMessage($messageBody, $messageTitle, $severity, $storeInSession);
    }

    /**
     * @param $argument
     * @param array $properties
     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException
     */
    protected function setDateTimeConverterOptions($argument, array $properties)
    {
        $propertyMappingConfiguration = $this->arguments->getArgument($argument)->getPropertyMappingConfiguration();
        foreach ($properties as $property) {
            $propertyMappingConfiguration->forProperty($property)->setTypeConverterOption(
                DateTimeConverter::class,
                DateTimeConverter::CONFIGURATION_DATE_FORMAT,
                'Y-m-d H:i:s'
            );
        }
    }

    /**
     * @param int $requiredUserGroup UID of the user group
     * @return bool
     */
    protected function backendUserHasUserGroup($requiredUserGroup=0)
    {
        if (Configuration::AUTH_NONE === $requiredUserGroup) {
            return true;
        }
        foreach ($this->backendUser->userGroups as $userGroup) {
            if ((int)$userGroup['uid'] === $requiredUserGroup) {
                return true;
            }
        }
        return false;
    }
}
