<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use Doctrine\Common\Util\Debug;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Class AbstractProtectedBeController
 */
abstract class AbstractProtectedBeController extends AbstractBeController
{
    /**
     *
     */
    protected function initializeAction()
    {
        if (!$this->backendUserHasUserGroup($this->getRequiredUserGroup())) {
            $this->addFlashMessage(
                LocalizationUtility::translate('controller.be.protected_controller.no_permission', 'election'),
                LocalizationUtility::translate('controller.be.protected_controller.error', 'election'),
                AbstractMessage::ERROR
            );
            $this->redirect(BeDashboardController::ACTION_INDEX, BeDashboardController::CONTROLLER_NAME);
        }
    }

    /**
     * @return int
     */
    abstract protected function getRequiredUserGroup();

    protected function errorAction()
    {
        $this->clearCacheOnError();

        $message="";
        foreach ($this->arguments->getValidationResults()->getFlattenedErrors() as $propertyPath => $errors) {
            foreach ($errors as $error) {
                $message .= PHP_EOL. 'Error for ' . $propertyPath . ': ' . $error->render();
            }
        }

        $errorFlashMessage = $this->getErrorFlashMessage();
        if ($errorFlashMessage !== FALSE) {
            $errorFlashMessageObject = new \TYPO3\CMS\Core\Messaging\FlashMessage(
                $errorFlashMessage." ".$message,
                '',
                \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR
            );
            $this->controllerContext->getFlashMessageQueue()->enqueue($errorFlashMessageObject);
        }
        $referringRequest = $this->request->getReferringRequest();
        if ($referringRequest !== NULL) {
            $originalRequest = clone $this->request;
            $this->request->setOriginalRequest($originalRequest);
            $this->request->setOriginalRequestMappingResults($this->arguments->getValidationResults());
            $this->forward($referringRequest->getControllerActionName(), $referringRequest->getControllerName(), $referringRequest->getControllerExtensionName(), $referringRequest->getArguments());
        }
        $message = 'An error occurred while trying to call ' . get_class($this) . '->' . $this->actionMethodName . '().' . PHP_EOL;
        return $message;

    }

}
