<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\Election\Domain\Model\Configuration;

/**
 * Class BeConfigurationController
 */
class BeConfigurationController extends AbstractProtectedBeController
{
    const CONTROLLER_NAME = 'BeConfiguration';
    const ACTION_EDIT = 'edit';
    const ACTION_UPDATE = 'update';

    /**
     * @var \TYPO3\CMS\Extbase\Domain\Repository\BackendUserGroupRepository
     * @inject
     */
    protected $backendUserGroupRepository = null;

    /**
     *
     */
    public function editAction()
    {
        if (null === $this->configuration) {
            $this->configuration = new Configuration();
        }
        $this->view->assign('configuration', $this->configuration);
        $this->view->assign('beUserGroups', $this->backendUserGroupRepository->findAll());
    }

    /**
     * @param Configuration $configuration
     */
    public function updateAction(Configuration $configuration)
    {
        $this->configurationRepository->update($configuration);
        $this->redirect(BeDashboardController::ACTION_INDEX, BeDashboardController::CONTROLLER_NAME);
    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return implode(',', [self::ACTION_EDIT, self::ACTION_UPDATE]);
    }

    /**
     * @return int
     */
    protected function getRequiredUserGroup()
    {
        if (null === $this->configuration) {
            return Configuration::AUTH_NONE;
        }
        return $this->configuration->getAdministrationGroup();
    }
}
