<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\Election\Domain\Repository\ElectionCircularRepository;
use TYPO3\Election\Domain\Repository\ElectorImportRepository;

/**
 * Dashboard controller
 */
class BeDashboardController extends AbstractBeController
{
    const CONTROLLER_NAME = 'BeDashboard';
    const ACTION_INDEX = 'index';

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionRepository
     * @inject
     */
    protected $electionRepository = null;

    /**
     *
     */
    public function indexAction()
    {
        $this->view->assignMultiple(
            [
                'elections' => $this->electionRepository->findAll(),
            ]
        );

        if ($this->configuration) {
            if ($this->configuration->isGroupConfigComplete()) {
                $electionManagerGroup = $this->configuration->getElectionManagerGroup();
            }
            else {
                $this->addFlashMessage(
                    LocalizationUtility::translate('view.be.dashboard.index.no_proper_configuration_msg_body', $this->extensionName),
                    LocalizationUtility::translate('view.be.dashboard.index.no_configuration_msg_title', $this->extensionName),
                    AbstractMessage::ERROR
                );
                if ($GLOBALS['BE_USER']->isAdmin()) {
                    $this->redirect('edit','BeConfiguration');
                } else {
                    $this->addFlashMessage(
                        LocalizationUtility::translate('view.be.dashboard.index.no_proper_configuration_msg_body_admin', $this->extensionName),
                        LocalizationUtility::translate('view.be.dashboard.index.no_configuration_msg_title', $this->extensionName),
                        AbstractMessage::ERROR
                    );
                }
            }
        }
        else {
            $this->addFlashMessage(
                LocalizationUtility::translate('view.be.dashboard.index.no_configuration_msg_body', $this->extensionName),
                LocalizationUtility::translate('view.be.dashboard.index.no_configuration_msg_title', $this->extensionName),
                AbstractMessage::ERROR
            );
        }

        if ($electionManagerGroup) {
            if ($this->backendUserHasUserGroup()) {
                $this->view->assign(
                    'electorImports',
                    $this->objectManager->get(ElectorImportRepository::class)->findAll()
                );
                $this->view->assign(
                    'electionCirculars',
                    $this->objectManager->get(ElectionCircularRepository::class)->findAll()
                );
            }
        }
        else {
            $this->addFlashMessage(
                LocalizationUtility::translate('view.be.dashboard.index.no_configuration_msg_body', $this->extensionName),
                LocalizationUtility::translate('view.be.dashboard.index.no_electionmanagergroup_msg_body', $this->extensionName),
                AbstractMessage::WARNING
            );
        }

    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return self::ACTION_INDEX;
    }
}
