<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\Election\Domain\Model\ElectionCircular;
use TYPO3\Election\Domain\Model\Elector;
use TYPO3\Election\Service\CircularService;
use TYPO3\Election\Service\ElectionInvitationService;
use TYPO3\Election\Utility\MailUtility;

/**
 * Class BeConfigurationController
 */
class BeElectionCircularController extends AbstractProtectedBeController
{
    const CONTROLLER_NAME = 'BeElectionCircular';
    const ACTION_LIST = 'list';
    const ACTION_NEW = 'new';
    const ACTION_CREATE = 'create';
    const ACTION_EDIT = 'edit';
    const ACTION_UPDATE = 'update';
    const ACTION_DELETE = 'delete';
    const ACTION_PREVIEW = 'preview';
    const ACTION_TEST_SEND = 'testSend';
    const ACTION_SEND = 'send';
    const ACTION_SEND_PREVIEW = 'sendPreview';

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionCircularRepository
     * @inject
     */
    protected $electionCircularRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionRepository
     * @inject
     */
    protected $electionRepository = null;

    /**
     *
     */
    public function listAction()
    {
        $this->view->assign('electionCirculars', $this->electionCircularRepository->findAll());
    }

    /**
     *
     */
    public function newAction()
    {
        $this->view->assign('electionCircular', $this->objectManager->get(ElectionCircular::class));
        $this->view->assign('elections', $this->electionRepository->findAll());
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function createAction(ElectionCircular $electionCircular)
    {
        $this->electionCircularRepository->add($electionCircular);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function editAction(ElectionCircular $electionCircular)
    {
        $this->view->assign('electionCircular', $electionCircular);
        $this->view->assign('elections', $this->electionRepository->findAll());
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function updateAction(ElectionCircular $electionCircular)
    {
        $this->electionCircularRepository->update($electionCircular);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function deleteAction(ElectionCircular $electionCircular)
    {
        $this->electionCircularRepository->remove($electionCircular);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function previewAction(ElectionCircular $electionCircular)
    {
        $this->view->assign('isPluginInstalled', $this->isPluginInstalled());
        $this->view->assign('electionCircular', $electionCircular);
    }

    /**
     * @param ElectionCircular $electionCircular
     * @param string $email
     */
    public function testSendAction(ElectionCircular $electionCircular, $email)
    {
        $elector = new Elector();
        $elector->setFirstName('[FIRST NAME]');
        $elector->setMiddleName('[MIDDLE NAME]');
        $elector->setLastName('[LAST NAME]');
        $elector->setEmail($email);
        $elector->setGender(rand(1, 2));
        $this->objectManager->get(MailUtility::class)->sendElectionCircularToElector(
            $electionCircular,
            $elector,
            $this->configuration
        );
        $this->redirect(
            self::ACTION_PREVIEW,
            self::CONTROLLER_NAME,
            'election',
            ['electionCircular' => $electionCircular]
        );
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function sendPreviewAction(ElectionCircular $electionCircular)
    {
        $this->view->assign('electionCircular', $electionCircular);
    }

    /**
     * @param ElectionCircular $electionCircular
     */
    public function sendAction(ElectionCircular $electionCircular)
    {
        $electionInvitationService = $this->objectManager->get(ElectionInvitationService::class);
        $electionInvitationService->inviteElectors($this->configuration, $electionCircular);
        $this->redirect(
            self::ACTION_SEND_PREVIEW,
            self::CONTROLLER_NAME,
            'election',
            ['electionCircular' => $electionCircular]
        );
    }

    /**
     * @return bool
     */
    protected function isPluginInstalled()
    {
        $where['CType']='list';
        $where['list_type']='election_pi1';
        $where['hidden']=0;
        // todo check also for enable fields
        #$extendedDeleteClause = BackendUtility::BEenableFields('tt_content');
        $count = $this->getDatabaseConnection()
            ->getConnectionForTable('tt_content')
            ->count('uid','tt_content',$where);


        return $count > 0;
    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return implode(
            ',',
            [
                self::ACTION_LIST,
                self::ACTION_NEW,
                self::ACTION_CREATE,
                self::ACTION_EDIT,
                self::ACTION_UPDATE,
                self::ACTION_DELETE,
                self::ACTION_PREVIEW,
                self::ACTION_TEST_SEND,
                self::ACTION_SEND,
                self::ACTION_SEND_PREVIEW,
            ]
        );
    }

    /**
     * @return int
     */
    protected function getRequiredUserGroup()
    {
        return $this->configuration->getElectionManagerGroup();
    }

    /**
     * @return ConnectionPool
     */
    protected function getDatabaseConnection()
    {
        return GeneralUtility::makeInstance(ConnectionPool::class);
    }
}
