<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\Election\Domain\Model\Election;

/**
 * Class BeConfigurationController
 */
class BeElectionController extends AbstractProtectedBeController
{
    const CONTROLLER_NAME = 'BeElection';
    const ACTION_LIST = 'list';
    const ACTION_NEW = 'new';
    const ACTION_CREATE = 'create';
    const ACTION_EDIT = 'edit';
    const ACTION_UPDATE = 'update';
    const ACTION_DELETE = 'delete';
    const ACTION_SUMMARY = 'summary';

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionRepository
     * @inject
     */
    protected $electionRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectorateRepository
     * @inject
     */
    protected $electorateRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\NomineeRepository
     * @inject
     */
    protected $nomineeRepository = null;

    /**
     *
     */
    public function listAction()
    {
        $this->view->assign('elections', $this->electionRepository->findAll());
    }

    /**
     *
     */
    public function newAction()
    {
        $this->view->assign('election', $this->objectManager->get(Election::class));
        $this->view->assign('electorates', $this->electorateRepository->findAll());
        $this->view->assign('nominees', $this->nomineeRepository->findAll());
    }

    /**
     * @param Election $election
     */
    public function editAction(Election $election)
    {
        $this->view->assign('election', $election);
        $this->view->assign('electorates', $this->electorateRepository->findAll());
        $this->view->assign('nominees', $this->nomineeRepository->findAll());
    }

    /**
     * @param Election $election
     */
    public function deleteAction(Election $election)
    {
        $this->electionRepository->remove($election);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     *
     */
    public function initializeUpdateAction()
    {
        $this->setDateTimeConverterOptions('election', ['startDate', 'endDate']);
    }

    /**
     * @param Election $election
     */
    public function updateAction(Election $election)
    {
        $this->electionRepository->update($election);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     *
     */
    public function initializeCreateAction()
    {
        $this->setDateTimeConverterOptions('election', ['startDate', 'endDate']);
    }

    /**
     * @param Election $election
     */
    public function createAction(Election $election)
    {
        $this->electionRepository->add($election);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Election $election
     */
    public function summaryAction(Election $election)
    {
        $this->view->assign('election', $election);
    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return implode(
            ',',
            [
                self::ACTION_LIST,
                self::ACTION_NEW,
                self::ACTION_CREATE,
                self::ACTION_UPDATE,
                self::ACTION_EDIT,
                self::ACTION_DELETE,
                self::ACTION_SUMMARY,
            ]
        );
    }

    /**
     * @return int
     */
    protected function getRequiredUserGroup()
    {
        return $this->configuration->getElectionManagerGroup();
    }
}
