<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\Election\Domain\Model\Dto\ElectorImport as ElectorImportDto;
use TYPO3\Election\Domain\Model\Election;
use TYPO3\Election\Domain\Model\Elector;
use TYPO3\Election\Domain\Model\ElectorImport;
use TYPO3\Election\Utility\CsvUtility;
use TYPO3\Election\Utility\FileUtility;

/**
 * Class BeConfigurationController
 */
class BeElectorController extends AbstractProtectedBeController
{
    const CONTROLLER_NAME = 'BeElector';
    const ACTION_NEW = 'new';
    const ACTION_CREATE = 'create';
    const ACTION_EDIT = 'edit';
    const ACTION_UPDATE = 'update';
    const ACTION_DELETE = 'delete';
    const ACTION_LIST = 'list';
    const ACTION_IMPORT = 'import';
    const ACTION_EXPORT = 'export';
    const ACTION_UPLOAD = 'upload';
    const ACTION_PROCESS_IMPORT = 'processImport';
    const ACTION_OVERWRITE_ALL = 'overwriteAll';

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectorateRepository
     * @inject
     */
    protected $electorateRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectorRepository
     * @inject
     */
    protected $electorRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectorImportRepository
     * @inject
     */
    protected $electorImportRepository = null;

    /**
     *
     */
    public function newAction()
    {
        $this->view->assign('elector', $this->objectManager->get(Elector::class));
        $this->view->assign('electorates', $this->electorateRepository->findAll());
    }

    /**
     * @param Elector $elector
     */
    public function createAction(Elector $elector)
    {
        $this->electorRepository->add($elector);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Elector $elector
     */
    public function editAction(Elector $elector)
    {
        $this->view->assign('elector', $elector);
        $this->view->assign('electorates', $this->electorateRepository->findAll());
    }

    /**
     * @param Elector $elector
     */
    public function updateAction(Elector $elector)
    {
        $this->electorRepository->update($elector);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Elector $elector
     */
    public function deleteAction(Elector $elector)
    {
        $this->electorRepository->remove($elector);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Election|null $election Optional Election which any imported Elector will be assigned to
     */
    public function importAction(Election $election = null)
    {
        $this->view->assign('electorImport', new ElectorImportDto($election, $this->electorateRepository->findAll()));
    }

    /**
     *
     */
    public function listAction()
    {
        $this->view->assign('electors', $this->electorRepository->findAll());
    }

    /**
     *
     */
    public function exportAction()
    {
        FileUtility::pushFile(
            $this->objectManager->get(CsvUtility::class)->convertFromArray(
                $this->electorRepository->findAll()->toArray(),
                array('firstName', 'middleName', 'lastName', 'gender', 'email')
            )
        );
    }

    /**
     * @param ElectorImportDto $electorImportDto
     */
    public function uploadAction(ElectorImportDto $electorImportDto)
    {
        if (FileUtility::isUploadedFileGood($electorImportDto->getImportInformation())) {
            $this->electorImportRepository->removeAll();
            $fileContents = FileUtility::getUploadedFileContents($electorImportDto->getImportInformation());
            $electorImports = $this->objectManager->get(CsvUtility::class)->convertToArray($fileContents);
            foreach ($electorImports as $electorImport) {
                $electorImportObject = new ElectorImport();
                $electorImportObject->setFromArray($electorImport);
                $electorImportObject->setElectorate($electorImportDto->getChosenElectorate());
                $this->electorImportRepository->add($electorImportObject);
            }
        }
        $this->redirect(self::ACTION_PROCESS_IMPORT);
    }

    /**
     *
     */
    public function processImportAction()
    {
        $this->view->assign('electorImports', $this->electorImportRepository->findAll());
    }

    /**
     *
     */
    public function overwriteAllAction()
    {
        $this->electorRepository->removeAll();
        /** @var ElectorImport $electorImport */
        foreach ($this->electorImportRepository->findAll() as $electorImport) {
            $elector = $this->objectManager->get(Elector::class);
            $elector->setFirstName($electorImport->getFirstName());
            $elector->setMiddleName($electorImport->getMiddleName());
            $elector->setLastName($electorImport->getLastName());
            $elector->setGender($electorImport->getGender());
            $elector->setEmail($electorImport->getEmail());
            $elector->addElectorate($electorImport->getElectorate());
            $this->electorRepository->add($elector);
            $this->electorImportRepository->remove($electorImport);
        }
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return implode(
            ',',
            [
                self::ACTION_NEW,
                self::ACTION_CREATE,
                self::ACTION_EDIT,
                self::ACTION_UPDATE,
                self::ACTION_DELETE,
                self::ACTION_IMPORT,
                self::ACTION_LIST,
                self::ACTION_EXPORT,
                self::ACTION_UPLOAD,
                self::ACTION_PROCESS_IMPORT,
                self::ACTION_OVERWRITE_ALL,
            ]
        );
    }

    /**
     * @return int
     */
    protected function getRequiredUserGroup()
    {
        return $this->configuration->getElectionManagerGroup();
    }
}
