<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\Election\Domain\Model\Electorate;

/**
 * Class BeElectorateController
 */
class BeElectorateController extends AbstractProtectedBeController
{
    const CONTROLLER_NAME = 'BeElectorate';
    const ACTION_LIST = 'list';
    const ACTION_NEW = 'new';
    const ACTION_EDIT= 'edit';
    const ACTION_CREATE = 'create';
    const ACTION_UPDATE = 'update';
    const ACTION_CREATE_AND_IMPORT_ELECTORS = 'createAndImportElectors';
    const ACTION_UPDATE_AND_IMPORT_ELECTORS = 'updateAndImportElectors';

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectorateRepository
     * @inject
     */
    protected $electorateRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectorRepository
     * @inject
     */
    protected $electorRepository = null;

    /**
     *
     */
    public function listAction()
    {
        $this->view->assign('electorates', $this->electorateRepository->findAll());
    }

    /**
     *
     */
    public function newAction()
    {
        $this->view->assign('electorate', $this->objectManager->get(Electorate::class));
        $this->view->assign('electors', $this->electorRepository->findAll());
    }

    /**
     * @param Electorate $electorate
     */
    public function createAction(Electorate $electorate)
    {
        $this->electorateRepository->add($electorate);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Electorate $electorate
     */
    public function editAction(Electorate $electorate)
    {
        $this->view->assign('electorate', $electorate);
        $this->view->assign('electors', $this->electorRepository->findAll());
    }

    /**
     * @param Electorate $electorate
     */
    public function updateAction(Electorate $electorate) {
        $this->electorateRepository->update($electorate);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Electorate $electorate
     */
    public function createAndImportElectorsAction(Electorate $electorate)
    {
        $this->electorateRepository->add($electorate);
        $this->redirect(
            BeElectorController::ACTION_IMPORT,
            BeElectorController::CONTROLLER_NAME,
            'election',
            ['electorate' => $electorate]
        );
    }

    /**
     * @param Electorate $electorate
     */
    public function updateAndImportElectorsAction(Electorate $electorate)
    {
        $this->electorateRepository->update($electorate);
        $this->redirect(
            BeElectorController::ACTION_IMPORT,
            BeElectorController::CONTROLLER_NAME,
            'election',
            ['electorate' => $electorate]
        );
    }

    /**
     * @return int
     */
    protected function getRequiredUserGroup()
    {
        return $this->configuration->getElectionManagerGroup();
    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return implode(
            ',',
            [
                self::ACTION_LIST,
                self::ACTION_NEW,
                self::ACTION_CREATE,
                self::ACTION_EDIT,
                self::ACTION_UPDATE,
                self::ACTION_CREATE_AND_IMPORT_ELECTORS,
                self::ACTION_UPDATE_AND_IMPORT_ELECTORS,
            ]
        );
    }
}
