<?php
namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\Election\Domain\Model\Nominee;
use TYPO3\Election\Property\TypeConverter\UploadedFileReferenceConverter;

/**
 * Class BeConfigurationController
 */
class BeNomineeController extends AbstractProtectedBeController
{
    const CONTROLLER_NAME = 'BeNominee';
    const ACTION_LIST = 'list';
    const ACTION_NEW = 'new';
    const ACTION_CREATE = 'create';
    const ACTION_EDIT = 'edit';
    const ACTION_UPDATE = 'update';
    const ACTION_HIDE = 'hide';
    const ACTION_SHOW = 'show';

    /**
     * @var \TYPO3\Election\Domain\Repository\NomineeRepository
     * @inject
     */
    protected $nomineeRepository = null;

    /**
     *
     */
    public function listAction()
    {
        $this->view->assign('nominees', $this->nomineeRepository->findAll());
    }

    /**
     *
     */
    public function newAction()
    {
        $this->view->assign('nominee', $this->objectManager->get(Nominee::class));
    }

    /**
     *
     */
    public function initializeCreateAction()
    {
        $this->setTypeConverterConfigurationForImageUpload('nominee', 'image');
    }

    /**
     * @param Nominee $nominee
     */
    public function createAction(Nominee $nominee)
    {
        $this->nomineeRepository->add($nominee);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Nominee $nominee
     */
    public function editAction(Nominee $nominee)
    {
        $this->view->assign('nominee', $nominee);
    }

    /**
     * @param Nominee $nominee
     */
    public function updateAction(Nominee $nominee)
    {
        $this->nomineeRepository->update($nominee);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Nominee $nominee
     */
    public function hideAction(Nominee $nominee)
    {
        $nominee->setPublic(false);
        $this->nomineeRepository->update($nominee);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @param Nominee $nominee
     */
    public function showAction(Nominee $nominee)
    {
        $nominee->setPublic(true);
        $this->nomineeRepository->update($nominee);
        $this->redirect(self::ACTION_LIST);
    }

    /**
     * @return string
     */
    public static function getActionsForModuleConfiguration()
    {
        return implode(
            ',',
            [
                self::ACTION_LIST,
                self::ACTION_NEW,
                self::ACTION_CREATE,
                self::ACTION_EDIT,
                self::ACTION_UPDATE,
                self::ACTION_HIDE,
                self::ACTION_SHOW,
            ]
        );
    }

    /**
     * @return int
     */
    protected function getRequiredUserGroup()
    {
        return $this->configuration->getElectionManagerGroup();
    }

    /**
     * @param string $argumentName
     * @param string $property
     */
    protected function setTypeConverterConfigurationForImageUpload($argumentName, $property)
    {
        $this->arguments->getArgument($argumentName)
            ->getPropertyMappingConfiguration()
            ->forProperty($property)
            ->setTypeConverter($this->objectManager->get(UploadedFileReferenceConverter::class))
            ->setTypeConverterOptions(
                UploadedFileReferenceConverter::class,
                [
                    UploadedFileReferenceConverter::CONFIGURATION_ALLOWED_EXTENSIONS => $this->getAllowedExtensions(),
                    UploadedFileReferenceConverter::CONFIGURATION_UPLOAD_FOLDER => $this->getUploadFolder(),
                ]
            );
    }

    /**
     * @return string
     */
    protected function getAllowedExtensions()
    {
        return $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'];
    }

    /**
     * @return string
     */
    protected function getUploadFolder()
    {
        return '1:/tx_election/';
    }
}
