<?php
namespace TYPO3\Election\Domain\Model;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Class AbstractPerson
 */
class AbstractPerson extends AbstractEntity
{
    const GENDER_NONE = 0;
    const GENDER_MALE = 1;
    const GENDER_FEMALE = 2;

    /**
     * @var string
     */
    protected $firstName = '';

    /**
     * @var string
     */
    protected $middleName = '';

    /**
     * @var string
     */
    protected $lastName = '';

    /**
     * @var int
     */
    protected $gender = self::GENDER_NONE;

    /**
     * @var string
     */
    protected $email = '';

    /**
     * @return string
     */
    public function getFirstName()
    {
        return $this->firstName;
    }

    /**
     * @param string $firstName
     */
    public function setFirstName($firstName)
    {
        $this->firstName = $firstName;
    }

    /**
     * @return string
     */
    public function getMiddleName()
    {
        return $this->middleName;
    }

    /**
     * @param string $middleName
     */
    public function setMiddleName($middleName)
    {
        $this->middleName = $middleName;
    }

    /**
     * @return string
     */
    public function getLastName()
    {
        return $this->lastName;
    }

    /**
     * @param string $lastName
     */
    public function setLastName($lastName)
    {
        $this->lastName = $lastName;
    }

    /**
     * @return int
     */
    public function getGender()
    {
        return $this->gender;
    }

    /**
     * @param int $gender
     */
    public function setGender($gender)
    {
        $this->gender = $gender;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * @return string
     */
    public function getFullName()
    {
        return implode(' ', [$this->firstName, $this->middleName, $this->lastName]);
    }

    /**
     * @return array
     */
    public function getTranslatedGenderOptions()
    {
        return [
            self::GENDER_NONE => LocalizationUtility::translate(
                'model.abstract_person.field.gender.' . self::GENDER_NONE,
                'election'
            ),
            self::GENDER_MALE => LocalizationUtility::translate(
                'model.abstract_person.field.gender.' . self::GENDER_MALE,
                'election'
            ),
            self::GENDER_FEMALE => LocalizationUtility::translate(
                'model.abstract_person.field.gender.' . self::GENDER_FEMALE,
                'election'
            ),
        ];
    }

    /**
     * @return string
     */
    public function getSalutation()
    {
        return LocalizationUtility::translate('model.abstract_person.salutation.' . $this->gender, 'election');
    }
}
