<?php
namespace TYPO3\Election\Domain\Model;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

class Configuration extends AbstractEntity
{
    const AUTH_NONE = 0;

    /**
     * Needed for CRUD actions of the Configuration
     *
     * @var int
     */
    protected $administrationGroup = 0;

    /**
     * Needed for CRUD actions of Electors, Electorates, Nominees and Elections
     *
     * @var int
     */
    protected $electionManagerGroup = 0;

    /**
     * Needed for CRUD actions of Questions and Polls
     *
     * @var int
     */
    protected $pollManagerGroup = 0;

    /**
     * @var int
     */
    protected $pluginPid = 0;

    /**
     * @var int
     */
    protected $numberOfMails = 10;

    /**
     * @var string
     */
    protected $fromEmail = '';

    /**
     * @var string
     */
    protected $fromName = '';

    /**
     * @var bool
     */
    protected $debug = false;

    /**
     * @var string
     */
    protected $testEmail = '';

    /**
     * @return int
     */
    public function getAdministrationGroup()
    {
        return $this->administrationGroup;
    }

    /**
     * @param int $administrationGroup
     */
    public function setAdministrationGroup($administrationGroup)
    {
        $this->administrationGroup = $administrationGroup;
    }

    /**
     * @return int
     */
    public function getElectionManagerGroup()
    {
        return $this->electionManagerGroup;
    }

    /**
     * @param int $electionManagerGroup
     */
    public function setElectionManagerGroup($electionManagerGroup)
    {
        $this->electionManagerGroup = $electionManagerGroup;
    }

    /**
     * @return int
     */
    public function getPollManagerGroup()
    {
        return $this->pollManagerGroup;
    }

    /**
     * @param int $pollManagerGroup
     */
    public function setPollManagerGroup($pollManagerGroup)
    {
        $this->pollManagerGroup = $pollManagerGroup;
    }

    /**
     * @return int
     */
    public function getPluginPid()
    {
        return $this->pluginPid;
    }

    /**
     * @param int $pluginPid
     */
    public function setPluginPid($pluginPid)
    {
        $this->pluginPid = $pluginPid;
    }

    /**
     * @return int
     */
    public function getNumberOfMails()
    {
        return $this->numberOfMails;
    }

    /**
     * @param int $numberOfMails
     */
    public function setNumberOfMails($numberOfMails)
    {
        $this->numberOfMails = $numberOfMails;
    }

    /**
     * @return string
     */
    public function getFromEmail()
    {
        return $this->fromEmail;
    }

    /**
     * @param string $fromEmail
     */
    public function setFromEmail($fromEmail)
    {
        $this->fromEmail = $fromEmail;
    }

    /**
     * @return string
     */
    public function getFromName()
    {
        return $this->fromName;
    }

    /**
     * @param string $fromName
     */
    public function setFromName($fromName)
    {
        $this->fromName = $fromName;
    }

    /**
     * @return boolean
     */
    public function isDebug()
    {
        return $this->debug;
    }

    /**
     * @param boolean $debug
     */
    public function setDebug($debug)
    {
        $this->debug = $debug;
    }

    /**
     * @return string
     */
    public function getTestEmail()
    {
        return $this->testEmail;
    }

    /**
     * @param string $testEmail
     */
    public function setTestEmail($testEmail)
    {
        $this->testEmail = $testEmail;
    }

    public function isGroupConfigComplete() {
        if ($this->getAdministrationGroup() && $this->getElectionManagerGroup() && $this->getPollManagerGroup()) {
            return true;
        }
        return false;
    }

    /**
     * @return Configuration
     */
    public function getUpdatedObject()
    {
        $clone = new Configuration();
        $clone->setAdministrationGroup($this->getAdministrationGroup());
        $clone->setElectionManagerGroup($this->getElectionManagerGroup());
        $clone->setPollManagerGroup($this->getPollManagerGroup());
        $clone->setPluginPid($this->getPluginPid());
        $clone->setNumberOfMails($this->getNumberOfMails());
        $clone->setFromEmail($this->getFromEmail());
        $clone->setFromName($this->getFromName());
        $clone->setDebug($this->isDebug());
        $clone->setTestEmail($this->getTestEmail());
        return $clone;
    }
}
