<?php

namespace TYPO3\Election\Domain\Model;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

/**
 * Class Election
 */
class Election extends AbstractEntity
{
    /**
     * @var string
     */
    protected $title = '';

    /**
     * @var string
     */
    protected $description = '';

    /**
     * @var \DateTime
     */
    protected $startDate = null;

    /**
     * @var \DateTime
     */
    protected $endDate = null;

    /**
     * @var \TYPO3\Election\Domain\Model\Electorate
     */
    protected $electorate = null;

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Election\Domain\Model\Nominee>
     */
    protected $nominees = null;

    /**
     * @var int
     */
    protected $numberOfVotes = 0;

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Election\Domain\Model\ElectionVote>
     */
    protected $electionVotes = null;

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return \DateTime
     */
    public function getStartDate()
    {
        return $this->startDate;
    }

    /**
     * @param \DateTime $startDate
     */
    public function setStartDate($startDate)
    {
        $this->startDate = $startDate;
    }

    /**
     * @return \DateTime
     */
    public function getEndDate()
    {
        return $this->endDate;
    }

    /**
     * @param \DateTime $endDate
     */
    public function setEndDate($endDate)
    {
        $this->endDate = $endDate;
    }

    /**
     * @return \TYPO3\Election\Domain\Model\Electorate
     */
    public function getElectorate()
    {
        return $this->electorate;
    }

    /**
     * @param \VerteXVaaR\Election\Domain\Model\Electorate $electorate
     */
    public function setElectorate($electorate)
    {
        $this->electorate = $electorate;
    }

    /**
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage
     */
    public function getNominees()
    {
        return $this->nominees;
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage $nominees
     */
    public function setNominees($nominees)
    {
        $this->nominees = $nominees;
    }

    /**
     * @return int
     */
    public function getNumberOfVotes()
    {
        return $this->numberOfVotes;
    }

    /**
     * @param int $numberOfVotes
     */
    public function setNumberOfVotes($numberOfVotes)
    {
        $this->numberOfVotes = $numberOfVotes;
    }

    /**
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage
     */
    public function getElectionVotes()
    {
        return $this->electionVotes;
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage $electionVotes
     */
    public function setElectionVotes($electionVotes)
    {
        $this->electionVotes = $electionVotes;
    }

    /**
     * @return bool
     */
    public function isElectionFinished()
    {
        if ($this->getEndDate() === null) {
            return true;
        }
        if ($this->getEndDate()->getTimestamp() < time()) {
            return true;
        }

        return false;
    }

    /** @return bool
     */
    public function isElectionPlanned()
    {
        if ($this->getStartDate() === null) {
            return true;
        }
        if ($this->getStartDate()->getTimestamp() > time()) {
            return true;
        }
        return false;
    }

    /**
     * @return array
     */
    public function getVotingElectors()
    {
        $votingElectors = [];
        /** @var Elector $elector */
        foreach ($this->electorate->getElectors() as $elector) {
            $electionInvitation = $elector->getElectionInvitationForElection($this);
            if (false !== $electionInvitation) {
                if ($electionInvitation->isVoted()) {
                    $votingElectors[] = $elector;
                }
            }
        }

        return $votingElectors;
    }

    /**
     * @return array
     */
    public function getAbstinentElectors()
    {
        $votingElectors = [];
        /** @var Elector $elector */
        foreach ($this->electorate->getElectors() as $elector) {
            $electionInvitation = $elector->getElectionInvitationForElection($this);
            if (false !== $electionInvitation) {
                if (!$electionInvitation->isVoted()) {
                    $votingElectors[] = $elector;
                }
            }
        }

        return $votingElectors;
    }

    /**
     * @return array
     */
    public function getUninvitedElectors()
    {
        $uninvitedElectors = [];
        /** @var Elector $elector */
        foreach ($this->electorate->getElectors() as $elector) {
            $electionInvitation = $elector->getElectionInvitationForElection($this);
            if (false === $electionInvitation) {
                $uninvitedElectors[] = $elector;
            }
        }

        return $uninvitedElectors;
    }

    /**
     * @return array
     */
    public function getElectedNomineesAndVotes()
    {
        $nominees = [];
        /** @var ElectionVote $electionVote */
        foreach ($this->getElectionVotes() as $electionVote) {
            $nominee = $electionVote->getNominee();
            $uid = $nominee->getUid();
            if (isset($nominees[$uid])) {
                $nominees[$uid]['votes']++;
            } else {
                $nominees[$uid]['votes'] = 1;
                $nominees[$uid]['nominee'] = $nominee;
            }
            if ($this->getElectionVotes() > 0) {
                $nominees[$uid]['relativevotes'] =
                    round($nominees[$uid]['votes'] / count($this->getElectionVotes()) * 100, 2);
            }
        }
        uasort(
            $nominees,
            function ($a, $b) {
                if ($a['votes'] > $b['votes']) {
                    return -1;
                }
                if ($a['votes'] < $b['votes']) {
                    return 1;
                }

                return 0;
            }
        );

        return $nominees;
    }
}
