<?php
namespace TYPO3\Election\Domain\Model;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\TimeTracker\TimeTracker;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use TYPO3\CMS\Extbase\Security\Cryptography\HashService;
use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;
use TYPO3\Election\Controller\FeElectionController;
use TYPO3\Election\Domain\Repository\ConfigurationRepository;

/**
 * Class ElectionCircular
 */
class ElectionCircular extends AbstractEntity
{
    /**
     * @var string
     */
    protected $subject = '';

    /**
     * @var string
     */
    protected $body = '';

    /**
     * @var \TYPO3\Election\Domain\Model\Election
     */
    protected $election = null;

    /**
     * @return string
     */
    public function getSubject()
    {
        return $this->subject;
    }

    /**
     * @param string $subject
     */
    public function setSubject($subject)
    {
        $this->subject = $subject;
    }

    /**
     * @return string
     */
    public function getBody()
    {
        return $this->body;
    }

    /**
     * @param string $body
     */
    public function setBody($body)
    {
        $this->body = $body;
    }

    /**
     * @return Election
     */
    public function getElection()
    {
        return $this->election;
    }

    /**
     * @param Election $election
     */
    public function setElection($election)
    {
        $this->election = $election;
    }

    /**
     *
     */
    public function getDummyBody()
    {
        $dummyElector = new Elector();
        $dummyElector->setFirstName('Kasper');
        $dummyElector->setLastName('Skårhøj');
        $dummyElector->setGender(Elector::GENDER_MALE);
        $dummyElector->setEmail('kasper@typo3.org');
        $dummyElectionInvitation = new ElectionInvitation();
        $dummyElectionInvitation->setElector($dummyElector);
        $dummyElectionInvitation->_setProperty('uid', -1);
        $dummyElectionInvitation->setSecret('foooBAAAAArrrrr');
        return $this->getRenderedBody(
            GeneralUtility::makeInstance(ObjectManager::class)->get(ConfigurationRepository::class)->findCurrent(),
            $dummyElectionInvitation
        );
    }

    /**
     * @param Configuration $configuration
     * @param ElectionInvitation $electionInvitation
     * @return string
     */
    public function getRenderedBody(Configuration $configuration, ElectionInvitation $electionInvitation)
    {
        $elector = $electionInvitation->getElector();
        return str_replace(
            [
                '{elector.salutation}',
                '{elector.firstName}',
                '{elector.middleName}',
                '{elector.lastName}',
                '{elector.fullName}',
                '{elector.email}',
                '{electionCircular.link}',
            ],
            [
                $elector->getSalutation(),
                $elector->getFirstName(),
                $elector->getMiddleName(),
                $elector->getLastName(),
                $elector->getFullName(),
                $elector->getEmail(),
                $this->getLink($configuration, $electionInvitation),
            ],
            $this->body
        );
    }

    /**
     * @param Configuration $configuration
     * @param ElectionInvitation $electionInvitation
     * @return string
     * @throws \TYPO3\CMS\Core\Error\Http\ServiceUnavailableException
     * @throws \TYPO3\CMS\Extbase\Security\Exception\InvalidArgumentForHashGenerationException
     */
    protected function getLink(Configuration $configuration, ElectionInvitation $electionInvitation)
    {
        $objectManager = GeneralUtility::makeInstance(ObjectManager::class);
        $hashService = $objectManager->get(HashService::class);

        if (!isset($GLOBALS['TT'])) {
            $GLOBALS['TT'] = GeneralUtility::makeInstance(TimeTracker::class);
        }
        if (!isset($GLOBALS['TSFE'])) {
            $GLOBALS['TSFE'] = GeneralUtility::makeInstance(
                TypoScriptFrontendController::class,
                $GLOBALS['TYPO3_CONF_VARS'],
                $configuration->getPluginPid(),
                0
            );
            $GLOBALS['TSFE']->config = ['config' => []];
            $GLOBALS['TSFE']->initFEuser();
            $GLOBALS['TSFE']->fetch_the_id();
            $GLOBALS['TSFE']->initTemplate();
        }

        $uriBuilder = $objectManager->get(UriBuilder::class);
        $uriBuilder->setTargetPageUid($configuration->getPluginPid());
        $uriBuilder->setCreateAbsoluteUri(true);
        $uriBuilder->setUseCacheHash(false);
        $uriBuilder->uriFor(
            FeElectionController::ACTION_VOTE,
            [
                'electionInvitation' => $electionInvitation,
                'hmac' => $hashService->generateHmac(
                    $electionInvitation->getSecret() . $electionInvitation->getElector()->getEmail()
                ),
            ],
            FeElectionController::CONTROLLER_NAME,
            'election',
            'pi1'
        );
        return $uriBuilder->buildFrontendUri();
    }

    /**
     * @return Elector[]
     */
    public function getRemainingReceivers()
    {
        $election = $this->getElection();
        $electors = $election->getElectorate()->getElectorsAsArray();
        $receivers = [];
        foreach ($electors as $elector) {
            if (!$elector->hasElectionInvitationForElection($election)) {
                $receivers[] = $elector;
            }
        }
        return $receivers;
    }
}
