<?php
namespace TYPO3\Election\Domain\Model;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Extbase\Persistence\ObjectStorage;

/**
 * Class Elector
 */
class Elector extends AbstractPerson
{
    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Election\Domain\Model\Electorate>
     */
    protected $electorates = '';

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Election\Domain\Model\ElectionInvitation>
     */
    protected $electionInvitations = [];

    /**
     * Elector constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this->electorates = new ObjectStorage();
    }

    /**
     * @return \TYPO3\Election\Domain\Model\Electorate[]
     */
    public function getElectorates()
    {
        return $this->electorates;
    }

    /**
     * @param \TYPO3\Election\Domain\Model\Electorate[] $electorates
     */
    public function setElectorates($electorates)
    {
        $this->electorates = $electorates;
    }

    /**
     * @param Electorate $electorate
     */
    public function addElectorate(Electorate $electorate = null)
    {
        if (null !== $electorate) {
            $this->electorates->attach($electorate);
        }
    }

    /**
     * @return ElectionInvitation[]
     */
    public function getElectionInvitations()
    {
        return $this->electionInvitations;
    }

    /**
     * @param ObjectStorage $electionInvitations
     */
    public function setElectionInvitations($electionInvitations)
    {
        $this->electionInvitations = $electionInvitations;
    }

    /**
     * @param Election $election
     * @return bool
     */
    public function hasElectionInvitationForElection(Election $election)
    {
        foreach ($this->getElectionInvitations() as $electionInvitation) {
            if ($electionInvitation->getElectionCircular()->getElection() === $election) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param Election $election
     * @return bool|ElectionInvitation
     */
    public function getElectionInvitationForElection(Election $election)
    {
        foreach ($this->getElectionInvitations() as $electionInvitation) {
            if ($electionInvitation->getElectionCircular()->getElection() === $election) {
                return $electionInvitation;
            }
        }
        return false;
    }
}
