<?php
namespace TYPO3\Election\Domain\Validator;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\CMS\Extbase\Validation\Validator\AbstractValidator;
use TYPO3\Election\Domain\Model\Dto\ElectionVoting;

/**
 * Class ElectionVotingValidator
 */
class ElectionVotingValidator extends AbstractValidator
{
    /**
     * @param mixed $value
     */
    protected function isValid($value)
    {
        if ($value instanceof ElectionVoting) {
            if (count($value->getNominees()) > $value->getElectionInvitation()
                    ->getElectionCircular()
                    ->getElection()
                    ->getNumberOfVotes()
            ) {
                $this->addError(
                    LocalizationUtility::translate(
                        'validator.fe.election_voting.election_voting_object.to_many_nominees',
                        'election'
                    ),
                    1451137701
                );
                $value->setNominees(new ObjectStorage());
            }
        } else {
            $this->addError(
                LocalizationUtility::translate(
                    'validator.fe.election_voting.election_voting_object.not_an_object',
                    'election'
                ),
                1451142398
            );
        }
    }
}
