<?php
namespace TYPO3\Election\Service;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Mail\MailMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\Election\Domain\Model\Configuration;
use TYPO3\Election\Domain\Model\ElectionCircular;
use TYPO3\Election\Domain\Model\ElectionInvitation;
use TYPO3\Election\Domain\Model\Elector;
use TYPO3\CMS\Core\Crypto\Random;

/**
 * Class ElectionInvitationService
 */
class ElectionInvitationService
{
    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionInvitationRepository
     * @inject
     */
    protected $electionInvitationRepository = null;

    /**
     * @param Configuration $configuration
     * @param ElectionCircular $electionCircular
     */
    public function inviteElectors(Configuration $configuration, ElectionCircular $electionCircular)
    {
        $limit = $configuration->getNumberOfMails();
        $receiver = [];

        $possibleReceivers = $electionCircular->getRemainingReceivers();
        while (count($receiver) < $limit && count($possibleReceivers) > 0) {
            /** @var Elector $possibleReceiver */
            $possibleReceiver = array_shift($possibleReceivers);
            $receiver[] = $possibleReceiver;
            $electionInvitation = new ElectionInvitation();
            $electionInvitation->setElector($possibleReceiver);
            $electionInvitation->setElectionCircular($electionCircular);
            $electionInvitation->setSecret(GeneralUtility::makeInstance(Random::class)->generateRandomHexString(mt_rand(180,220)));
            $this->electionInvitationRepository->add($electionInvitation);
            $this->electionInvitationRepository->persistAll();
            $electionInvitation->setSuccess($this->sendElectionInvitation($configuration, $electionInvitation));
            $this->electionInvitationRepository->update($electionInvitation);
            $this->electionInvitationRepository->persistAll();
        }
    }

    /**
     * @param Configuration $configuration
     * @param ElectionInvitation $electionInvitation
     * @return bool
     */
    protected function sendElectionInvitation(
        Configuration $configuration,
        ElectionInvitation $electionInvitation
    ) {
        $mail = GeneralUtility::makeInstance(MailMessage::class);

        $electionCircular = $electionInvitation->getElectionCircular();
        $elector = $electionInvitation->getElector();

        $mail->setSubject($electionCircular->getSubject());
        $mail->setFrom([$configuration->getFromEmail() => $configuration->getFromName()]);

        if ($configuration->isDebug()) {
            $mail->setTo([$configuration->getTestEmail() => $elector->getFullName()]);
        } else {
            $mail->setTo([$elector->getEmail() => $elector->getFullName()]);
        }

        $mail->setBody($electionCircular->getRenderedBody($configuration, $electionInvitation));
        return $mail->send() === 1;
    }
}
