<?php
namespace TYPO3\Election\Utility;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Extbase\Reflection\ObjectAccess;

/**
 * Class CsvUtility
 */
class CsvUtility
{
    /**
     * Pattern to match one line of a CSV file
     */
    const PATTERN_LINE = '/(?<=^|[,;])(((?:")(?P<quotedValue>[^"]*)(?:"))|((?<!")(?P<value>[^,;"]*)(?!")))(?=[,;]|$)/';

    /**
     * @param string $csv
     * @return array
     */
    public function convertToArray($csv)
    {
        $lines = explode(chr(10), $csv);
        $firstLine = trim(array_shift($lines));
        $columnNames = $this->convertLineToArray($firstLine);
        $columnCount = count($columnNames);
        $result = array();
        foreach ($lines as $line) {
            $lineArray = $this->convertLineToArray(trim($line));
            if (count($lineArray) !== $columnCount) {
                continue;
            }
            $result[] = array_combine($columnNames, $lineArray);
        }
        return $result;
    }

    /**
     * @param array $objects
     * @param array $propertyNames
     * @return string
     */
    public function convertFromArray(array $objects, array $propertyNames)
    {
        $result = '';
        $columnNames = array();
        foreach ($propertyNames as $propertyName) {
            $columnNames[] = $this->escapeValue($propertyName);
        }
        $result .= implode(';', $columnNames) . chr(10);

        foreach ($objects as $object) {
            $cells = array();
            foreach ($propertyNames as $propertyName) {
                $cells[] = $this->escapeValue(ObjectAccess::getPropertyPath($object, $propertyName));
            }
            $result .= implode(';', $cells) . chr(10);
        }
        return $result;
    }

    /**
     * @param string $value
     * @return string
     */
    protected function escapeValue($value)
    {
        if (strpos($value, ';') !== false) {
            $value = '"' . str_replace('"', '\"', $value) . '"';
        }
        return $value;
    }

    /**
     * @param string $line
     * @return array
     */
    protected function convertLineToArray($line)
    {
        $matches = array();
        if (preg_match_all(self::PATTERN_LINE, $line, $matches) < 1) {
            return false;
        }
        $result = array();
        foreach ($matches['value'] as $matchIndex => $match) {
            if (strlen($match) < 1) {
                $match = $matches['quotedValue'][$matchIndex];
            }
            $result[$matchIndex] = $match;
        }
        return $result;
    }
}
