<?php
namespace TYPO3\Election\Utility;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Mail\MailMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use TYPO3\Election\Domain\Model\Configuration;
use TYPO3\Election\Domain\Model\ElectionCircular;
use TYPO3\Election\Domain\Model\ElectionInvitation;
use TYPO3\Election\Domain\Model\Elector;
use TYPO3\Election\Domain\Repository\ElectionInvitationRepository;
use TYPO3\CMS\Core\Crypto\Random;

/**
 * Class MailUtility
 */
class MailUtility
{
    /**
     * @param ElectionCircular $electionCircular
     * @param Elector $elector
     * @param Configuration $configuration
     * @return bool
     */
    public function sendElectionCircularToElector(
        ElectionCircular $electionCircular,
        Elector $elector,
        Configuration $configuration
    ) {
        $mail = GeneralUtility::makeInstance(MailMessage::class);
        $mail->setSubject($electionCircular->getSubject());
        $mail->setFrom([$configuration->getFromEmail() => $configuration->getFromName()]);

        $electionInvitation = new ElectionInvitation();
        $electionInvitation->setElector($elector);
        $electionInvitation->setElectionCircular($electionCircular);
        $electionInvitation->setSecret(GeneralUtility::makeInstance(Random::class)->generateRandomHexString(mt_rand(180,220)));

        $electionInvitationRepository = GeneralUtility::makeInstance(ObjectManager::class)->get(ElectionInvitationRepository::class);
        $electionInvitationRepository->add($electionInvitation);
        $electionInvitationRepository->persistAll();


        if ($configuration->isDebug()) {
            $mail->setTo([$configuration->getTestEmail() => $elector->getFullName()]);
        } else {
            $mail->setTo([$elector->getEmail() => $elector->getFullName()]);
        }

        $mail->setBody($electionCircular->getRenderedBody($configuration, $electionInvitation));

        return $mail->send() === 1;
    }
}
